# Yield targeting

from definitions import *
from pyopus.design.yt import YieldTargeting
from pyopus.evaluator.aggregate import formatParameters
# If MPI is imported an application not using MPI will behave correctly
# (i.e. only slot 0 will run the program) even when started with mpirun/mpiexec
from pyopus.parallel.mpi import MPI
from pyopus.parallel.cooperative import cOS


if __name__=='__main__':
	atDesign={
		# Result of design across corners
		'c_out':    6.4025057781114802e-12,
		'diff_l':   3.4247816773735976e-07,
		'diff_w':   1.6738092946223754e-05,
		'load_l':   2.3668072815514710e-06,
		'load_w':   5.3240803355989389e-05,
		'mirr_l':   1.0157035640188838e-06,
		'mirr_ld':  1.6182520447950098e-06,
		'mirr_w':   5.8107861163445226e-05,
		'mirr_wd':  6.1242813846216839e-05,
		'mirr_wo':  6.8277846319352392e-05,
		'out_l':    4.1088626829554061e-07,
		'out_w':    8.1812469568552345e-05,
		'r_out':    5.1647907484619873e+02,
	}
	
	# Create corner with the Monte Carlo model
	corners={
		'mc': {
			'params': {}, 
			'modules': [ 'mc' ]
		}
	}
	
	# Remove transient analysis measures, design with DC and AC measures only
	#for name in ['tsetup', 'tsetdn', 'slewup', 'slewdn', 'overshup', 'overshdn' ]:
	#	del measures[name]
		
	# Worst case performances (skip area, phase_slope)
	wcList=list(measures.keys())
	wcList.sort()
	wcList.remove('area')
	wcList.remove('ph_slope')
	
	# Prepare parallel environment
	cOS.setVM(MPI(mirrorMap={'*':'.'}))
	
	# CBD settings
	initialCbdOptions={ 
		'debug': 1, 'method': 'global', 'stepTol': 1e-5, 
		'norms': { 'area': 100e-12, 'vgs_drv': 1e-3, 'vds_drv':1e-3 }, 
		'failurePenalty': 1e6, 
		'tradeoffs': 1e-6, 
		'stopWhenAllSatisfied': True, 
	}
	
	# CBD settings for second and later passes
	cbdOptions={}
	cbdOptions.update(initialCbdOptions)
	# cbdOptions['method']='local'
	
	wcOptions={
		'stepTol': 0.01, 
		'debug': 0, 
	}
	
	# 3-sigma target yield
	yt=YieldTargeting(
		designParams, statParams, opParams, 
		heads, analyses, measures, corners, variables=variables, 
		beta=3.0, wcSpecs=wcList, 
		# Comment out start with initial point (lo+hi)/2 and perform initial nominal design
		initial=atDesign, initialNominalDesign=False, 
		# Initial nominal optimization
		initialCbdOptions=initialCbdOptions, 
		# First pass optiomation
		firstPassCbdOptions=initialCbdOptions, 
		# Main optimization
		cbdOptions=cbdOptions, 
		# Worst-case analysis
		wcOptions=wcOptions, 
		debug=2, spawnerLevel=1
	)
	atDesign, agg, wc, anCount = yt()
	print(formatParameters(atDesign))
	print(wc.formatResults())
	if agg is not None:
		print(agg.formatResults())
	print(anCount)
	
	# Finalize cOS parallel environment
	cOS.finalize()
	
# Result 
#           c_out:   1.5906247662071053e-12
#          diff_l:   1.0647898762052580e-06
#          diff_w:   1.0954427560389812e-05
#          load_l:   1.9692895593955935e-06
#          load_w:   1.8559908362865259e-05
#          mirr_l:   9.7786062346348561e-07
#         mirr_ld:   2.8778986167864697e-06
#          mirr_w:   2.2130751573351460e-05
#         mirr_wd:   9.8183046192244794e-06
#         mirr_wo:   6.0483504290617440e-05
#           out_l:   5.8015162233222138e-07
#           out_w:   3.5473803134446091e-05
#           r_out:   2.1456835044861632e+03
